@@ -1,12 +1,70 @@
 #! /usr/bin/python3
 
-from datetime import date, datetime, timedelta
+import traceback
 import warnings
+from datetime import date, datetime, timedelta
+
 import pandas as pd
+import requests
 
 # https://docs.python.org/3/library/datetime.html#strftime-and-strptime-format-codes
 
+# HDDF = 'holidays.csv'
+HOLIDAYS = [
+    '01/01/2015', '01/02/2015', '01/03/2015',
+    '02/18/2015', '02/19/2015', '02/20/2015', '02/21/2015', '02/22/2015', '02/23/2015', '02/24/2015',
+    '04/06/2015', '05/01/2015', '06/20/2015', '06/21/2015', '06/22/2015',
+    '09/03/2015', '09/04/2015', '09/05/2015',
+    '10/01/2015', '10/02/2015', '10/03/2015', '10/04/2015', '10/05/2015', '10/06/2015', '10/07/2015' ] + \
+    [
+    '01/01/2016',
+    '02/07/2016', '02/08/2016', '02/09/2016', '02/10/2016', '02/11/2016', '02/12/2016', '02/13/2016',
+    '04/04/2016', '05/02/2016', '06/09/2016', '06/10/2016', '06/11/2016',
+    '09/15/2016', '09/16/2016', '09/17/2016',
+    '10/01/2016', '10/02/2016', '10/03/2016', '10/04/2016', '10/05/2016', '10/06/2016', '10/07/2016' ] + \
+    [
+    '01/01/2017', '01/02/2017', '01/27/2017', '01/28/2017', '01/29/2017', '01/30/2017', '01/31/2017',
+    '02/01/2017', '02/02/2017', '04/02/2017', '04/03/2017', '04/04/2017', '05/01/2017', '05/02/2017',
+    '05/03/2017', '05/28/2017', '05/29/2017', '05/30/2017', '10/01/2017', '10/02/2017', '10/03/2017',
+    '10/04/2017', '10/05/2017', '10/06/2017', '10/07/2017', '10/08/2017' ] + \
+    [
+    '01/01/2018', '02/15/2018', '02/16/2018', '02/17/2018', '02/18/2018', '02/19/2018', '02/20/2018',
+    '02/21/2018', '04/05/2018', '04/06/2018', '04/07/2018', '04/29/2018', '04/30/2018', '05/01/2018',
+    '06/16/2018', '06/17/2018', '06/18/2018', '09/24/2018', '10/01/2018', '10/02/2018', '10/03/2018',
+    '10/04/2018', '10/05/2018', '10/06/2018', '10/07/2018', "12/30/2018", "12/31/2018", "01/01/2019",
+    '02/04/2019', '02/05/2019', '02/06/2019', '02/07/2019', '02/08/2019', '02/09/2019', '02/10/2019', 
+    '04/05/2019', '04/06/2019', '04/07/2019',
+    '05/01/2019', '05/02/2019', '05/03/2019', '05/04/2019', '06/07/2019', '06/08/2019', '06/09/2019',
+    '09/13/2019', '09/14/2019', '09/15/2019', '10/01/2019', '10/02/2019', '10/03/2019',
+    '10/04/2019', '10/05/2019', '10/06/2019', '10/07/2019', '01/01/2020', '01/24/2020',
+    '01/25/2020', '01/26/2020', '01/27/2020', '01/28/2020', '01/29/2020', '01/30/2020',
+    '01/31/2020', '02/01/2020', '02/02/2020', '04/04/2020', '04/05/2020', '04/06/2020', '05/01/2020',
+    '05/02/2020', '05/03/2020', '05/04/2020', '05/05/2020', '06/25/2020', '06/26/2020', '06/27/2020',
+    '10/01/2020', '10/02/2020', '10/03/2020', '10/04/2020', '10/05/2020', '10/06/2020',
+    '10/07/2020', '10/08/2020', '01/01/2021', '01/02/2021', '01/03/2021', '02/11/2021', '02/12/2021',
+    '02/13/2021', '02/14/2021', '02/15/2021', '02/16/2021', '02/17/2021', '04/03/2021', '04/04/2021',
+    '04/05/2021', '05/01/2021', '05/02/2021', '05/03/2021', '05/04/2021', '05/05/2021',
+    '06/12/2021', '06/13/2021', '06/14/2021',
+    '09/19/2021', '09/20/2021', '09/21/2021', '10/01/2021', '10/02/2021', '10/03/2021', '10/04/2021',
+    '10/05/2021', '10/06/2021', '10/07/2021' ] + \
+    [
+    '01/01/2022', '01/02/2022', '01/03/2022',
+    '01/31/2022', '02/01/2022', '02/02/2022', '02/03/2022', '02/04/2022', '02/05/2022', '02/06/2022',
+    '04/03/2022', '04/04/2022', '04/05/2022',
+    '04/30/2022', '05/01/2022', '05/02/2022', '05/03/2022', '05/04/2022',
+    '06/03/2022', '06/04/2022', '06/05/2022',
+    '09/10/2022', '09/11/2022', '09/12/2022',
+    '10/01/2022', '10/02/2022', '10/03/2022', '10/04/2022', '10/05/2022', '10/06/2022', '10/07/2022' ]
+
 class Holidays:
+    SSEHolidays = HOLIDAYS
+    try:
+        r = requests.get('https://openapi.piasset.net/public/@holidays')
+        SSEHolidays = list(r.json())
+    except Exception as e:
+        # print(traceback.format_exc())
+        pass
+
     @classmethod
     def to_datetime(self, datestr:str) -> datetime:
         """
@@ -137,7 +195,7 @@ class Holidays:
         return ret
 
     @classmethod
-    def range_exp(self, start: str, end: str) -> pd.DataFrame:
+    def range_exp(self, start: str, end: str, cate: str="300") -> pd.DataFrame:
         """
         Return a pandas.DataFrame object with only one column named
         'exp' containing all the expirations between start and end.
@@ -174,10 +232,13 @@ class Holidays:
             raise ValueError(S)
         di = ds
         L = []
+        wd = 2 if cate.lower() == "etf"  else 4
+        dl = 22 if cate.lower() == "etf" else 15
+        du = 28 if cate.lower() == "etf" else 21
         while di <= de:
-            while di.weekday() != 4:
+            while di.weekday() != wd:
                 di = di + timedelta(days=1)
-            if 15 <= di.day <= 21:
+            if dl <= di.day <= du:
                 if self.tradingday(di):
                     L.append(di)
                 else:
@@ -219,63 +280,3 @@ class Holidays:
         ret = (pd.to_datetime(exps['exp'].values[0])).strftime('%Y-%m-%d')
         # print(ret, type(ret))
         return ret
-
-    SSEHolidays = [
-    '01/01/2015', '01/02/2015', '01/03/2015',
-    '02/18/2015', '02/19/2015', '02/20/2015', '02/21/2015', '02/22/2015', '02/23/2015', '02/24/2015',
-    '04/06/2015', '05/01/2015', '06/20/2015', '06/21/2015', '06/22/2015',
-    '09/03/2015', '09/04/2015', '09/05/2015',
-    '10/01/2015', '10/02/2015', '10/03/2015', '10/04/2015', '10/05/2015', '10/06/2015', '10/07/2015' ] + \
-    [
-    '01/01/2016',
-    '02/07/2016', '02/08/2016', '02/09/2016', '02/10/2016', '02/11/2016', '02/12/2016', '02/13/2016',
-    '04/04/2016', '05/02/2016', '06/09/2016', '06/10/2016', '06/11/2016',
-    '09/15/2016', '09/16/2016', '09/17/2016',
-    '10/01/2016', '10/02/2016', '10/03/2016', '10/04/2016', '10/05/2016', '10/06/2016', '10/07/2016' ] + \
-    [
-    '01/01/2017', '01/02/2017', '01/27/2017', '01/28/2017', '01/29/2017', '01/30/2017', '01/31/2017',
-    '02/01/2017', '02/02/2017', '04/02/2017', '04/03/2017', '04/04/2017', '05/01/2017', '05/02/2017',
-    '05/03/2017', '05/28/2017', '05/29/2017', '05/30/2017', '10/01/2017', '10/02/2017', '10/03/2017',
-    '10/04/2017', '10/05/2017', '10/06/2017', '10/07/2017', '10/08/2017' ] + \
-    [
-    '01/01/2018', '02/15/2018', '02/16/2018', '02/17/2018', '02/18/2018', '02/19/2018' ,'02/20/2018',
-    '02/21/2018', '04/05/2018', '04/06/2018', '04/07/2018', '04/29/2018', '04/30/2018', '05/01/2018',
-    '06/16/2018', '06/17/2018', '06/18/2018', '10/01/2018', '10/02/2018', '10/03/2018',
-    '10/04/2018', '10/05/2018', '10/06/2018', '10/07/2018', '02/04/2019', '02/05/2019', '02/06/2019',
-    '02/07/2019', '02/08/2019', '02/09/2019', '02/10/2019', '04/05/2019', '04/06/2019', '04/07/2019',
-    '05/01/2019', '05/02/2019', '05/03/2019', '05/04/2019', '06/07/2019', '06/08/2019', '06/09/2019',
-    '09/13/2019', '09/14/2019', '09/15/2019', '10/01/2019', '10/02/2019', '10/03/2019',
-    '10/04/2019', '10/05/2019', '10/06/2019', '10/07/2019', '01/01/2020', '01/24/2020',
-    '01/25/2020', '01/26/2020', '01/27/2020', '01/28/2020', '01/29/2020', '01/30/2020',
-    '01/31/2020', '02/01/2020', '02/02/2020', '04/04/2020', '04/05/2020', '04/06/2020', '05/01/2020',
-    '05/02/2020', '05/03/2020', '05/04/2020', '05/05/2020', '06/25/2020', '06/26/2020', '06/27/2020',
-    '10/01/2020', '10/02/2020', '10/03/2020', '10/04/2020', '10/05/2020', '10/06/2020',
-    '10/07/2020', '10/08/2020', '01/01/2021', '01/02/2021', '01/03/2021', '02/11/2021', '02/12/2021',
-    '02/13/2021', '02/14/2021', '02/15/2021', '02/16/2021', '02/17/2021', '04/03/2021', '04/04/2021',
-    '04/05/2021', '05/01/2021', '05/02/2021', '05/03/2021', '05/04/2021', '05/05/2021',
-    '06/12/2021', '06/13/2021', '06/14/2021',
-    '09/19/2021', '09/20/2021', '09/21/2021', '10/01/2021', '10/02/2021', '10/03/2021', '10/04/2021',
-    '10/05/2021', '10/06/2021', '10/07/2021' ] + \
-    [
-    '01/01/2022', '01/02/2022', '01/03/2022',
-    '01/31/2022', '02/01/2022', '02/02/2022', '02/03/2022', '02/04/2022', '02/05/2022', '02/06/2022',
-    '04/03/2022', '04/04/2022', '04/05/2022',
-    '04/30/2022', '05/01/2022', '05/02/2022', '05/03/2022', '05/04/2022',
-    '06/03/2022', '06/04/2022', '06/05/2022',
-    '09/10/2022', '09/11/2022', '09/12/2022',
-    '10/01/2022', '10/02/2022', '10/03/2022', '10/04/2022', '10/05/2022', '10/06/2022', '10/07/2022' ]
-
-
-
-
-if '__main__' == __name__:
-    dt1 = datetime(2021,5,6)
-    tm1 = "20210506"
-    print(type(dt1), Holidays.to_datetime(dt1), Holidays.tradingday(dt1), Holidays.prev_tradingday(dt1), Holidays.next_tradingday(dt1))
-    print(type(tm1), Holidays.to_datetime(tm1), Holidays.tradingday(tm1), Holidays.prev_tradingday(tm1), Holidays.next_tradingday(tm1))
-    dt2 = datetime(2021,9,30)
-    tm2 = "20210930"
-    print(type(dt2), Holidays.to_datetime(dt2), Holidays.tradingday(dt2), Holidays.prev_tradingday(dt2), Holidays.next_tradingday(dt2))
-    print(type(tm2), Holidays.to_datetime(tm2), Holidays.tradingday(tm2), Holidays.prev_tradingday(tm2), Holidays.next_tradingday(tm2))
-
-    print(Holidays.get_holidays())
\ No newline at end of file
